/*
	Created by:		Esposito Vittorio
	GitHub:			https://github.com/VittorioEsposito	
	
	J1850-PWM-Encoding-Library
	
	Arduino library which allows to communicate on J1850-PWM mode. 
	It works with both wire and radio transmissions. 
	You just have to choose your bit-rate (lower is better). 
	It doesn't have either a header nor a parity check. 
	These functions could be implemented at a higher protocol layer. 
	This library has been tested also with FSK module RTX-mid-868.
	
*/

#include <Arduino.h>
#include <PwmDecoder.h>
#include "avr/interrupt.h"

PwmDecoder::PwmDecoder(uint8_t RX_PIN)
{
	_RX_PIN = RX_PIN;
	pinMode(_RX_PIN, INPUT);
	pinMode(13, OUTPUT);
	flagDecoder = false;
}

void PwmDecoder::setRxPin(uint8_t RX_PIN)
{
	_RX_PIN = RX_PIN;
	pinMode(_RX_PIN, INPUT);
}

uint8_t PwmDecoder::getRxPin()
{
	return _RX_PIN;
}

void PwmDecoder::begin(uint32_t BITRATE)
{
	_BITRATE = BITRATE;
	_BIT_TIME = 1000000 / _BITRATE;  // uS
	_HALF_BIT_TIME = _BIT_TIME / 2;
	attachInterrupt(digitalPinToInterrupt(_RX_PIN), isrPwmDecoder, RISING);
}

static void PwmDecoder::isrPwmDecoder()
{
	timeOld = millis();						
	flagDecoder = true;
}

void PwmDecoder::receive()
{
	if (flagDecoder)
	{
		flagDecoder = false;
		delayMicroseconds(_HALF_BIT_TIME);
		if (digitalRead(_RX_PIN)) 
		{
			c = (c >> _count) + 1;
			c = (c << _count);
		}
		if (!_count) 
		{
			_count = 7;
			_RX_BUFFER[_BUFFER_INDEX] = c;
			_BUFFER_INDEX++;
			c = 0x00;
		} else {
			_count--;
		}
    }
}

bool PwmDecoder::isReceiving()			
{	
	// Check if buffer is full
	if (_BUFFER_INDEX > BUFFER_SIZE-1)
	{
		// Force it to quit this loop
		_BUFFER_INDEX = BUFFER_SIZE-1;
		return false;
	}
	// Check if millis has resetted
	if (millis() < timeOld) timeOld = millis();
	
	// Check the lost of synchronicity
	if (millis() > (timeOld + timeOut))
	{
		c = 0x00;
		_count = 7;
		return false;
	}
	else
	{
		return true;
	}
}

bool PwmDecoder::available()
{
	return (_BUFFER_INDEX > 0);
}

String PwmDecoder::readString()
{
	String str = "";
	int i;
	for (i=0; i<_BUFFER_INDEX; i++)
	{
		str += _RX_BUFFER[i];
		_RX_BUFFER[i] = 0x00;
	}
	_BUFFER_INDEX = 0;
	return str;
}