/*
	Created by:		Esposito Vittorio
	GitHub:			https://github.com/VittorioEsposito	
	
	J1850-PWM-Encoding-Library
	
	Arduino library which allows to communicate on J1850-PWM mode. 
	It works with both wire and radio transmissions. 
	You just have to choose your bit-rate (lower is better). 
	It doesn't have either a header nor a parity check. 
	These functions could be implemented at a higher protocol layer. 
	This library has been tested also with FSK module RTX-mid-868.
	
*/

#include <Arduino.h>
#include "avr/interrupt.h"

#define DEFAULT_RX_PIN 		3
#define BUFFER_SIZE			50

class PwmDecoder 
{
	static volatile bool flagDecoder;
	static volatile unsigned long timeOld;	
	
	private:
		uint8_t  _RX_PIN;
		uint32_t _BITRATE;
		uint32_t _BIT_TIME;
		uint32_t _HALF_BIT_TIME;
		char _RX_BUFFER[BUFFER_SIZE];
		uint16_t _BUFFER_INDEX = 0;
		int _count = 7;
		uint16_t timeOut = 500;		// mS	
		char c = 0x00;
	
	public:
		PwmDecoder(uint8_t RX_PIN);
		void setRxPin(uint8_t RX_PIN);
		uint8_t getRxPin();
		void begin(uint32_t BITRATE);
		
		static void isrPwmDecoder();
		void receive();
		bool isReceiving();					//
		bool available();
		String readString();
		
		
};

