/*
	Created by:		Esposito Vittorio
	GitHub:			https://github.com/VittorioEsposito	
	
	J1850-PWM-Encoding-Library
	
	Arduino library which allow to communicate on J1850-PWM mode. 
	It works with both wire and radio transmissions. 
	You just have to choose your bit-rate (lower is better). 
	It doesn't have either a header nor a parity check. 
	These functions could be implemented at a higher protocol layer. 
	This library has been tested also with FSK module RTX-mid-868.
	
*/

#include <Arduino.h>

#define DEFAULT_TX_PIN 		13

class PwmEncoder {
	
	public:
		PwmEncoder(uint8_t TX_PIN);
		PwmEncoder(uint8_t TX_PIN, uint8_t MODE_PIN, uint8_t EN_PIN);
		void setTxPin(uint8_t TX_PIN);
		void setEnablePin(uint8_t EN_PIN);
		void setTxRxModePin(uint8_t MODE_PIN);
		uint8_t getTxPin();
		void begin(uint32_t BITRATE);
		void powerDown();
		void powerUp();
		void enableTxMode();
		void enableRxMode();
		
		void println(String txt);
		void print(String txt);
		void print(char txt[]);
		void print(char c);
		void print(uint8_t n);
		void send (bool b);
	
	
	private:
		uint8_t _TX_PIN;
		uint8_t _EN_PIN;
		uint8_t _MODE_PIN;
		uint32_t _BITRATE;
		uint32_t BIT_TIME;
		uint32_t ONE_THIRD;
		uint32_t TWO_THIRD;
		
	
	
};